@@ -27,19 +27,22 @@ import java.io.FileWriter;
 import java.io.InputStreamReader;
 import java.io.IOException;
 import java.io.OutputStreamWriter;
-import java.io.PrintWriter;
 import java.io.Reader;
 import java.io.Writer;
-import java.util.Vector;
+import java.util.Iterator;
 import org.apache.tools.ant.BuildException;
-import org.apache.tools.ant.DirectoryScanner;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.Task;
 import org.apache.tools.ant.types.FileSet;
 import org.apache.tools.ant.types.RegularExpression;
+import org.apache.tools.ant.types.Resource;
+import org.apache.tools.ant.types.ResourceCollection;
 import org.apache.tools.ant.types.Substitution;
+import org.apache.tools.ant.types.resources.FileProvider;
+import org.apache.tools.ant.types.resources.Union;
 import org.apache.tools.ant.util.FileUtils;
 import org.apache.tools.ant.util.regexp.Regexp;
+import org.apache.tools.ant.util.regexp.RegexpUtil;
 
 /**
  * Performs regular expression string replacements in a text
@@ -53,17 +56,17 @@ import org.apache.tools.ant.util.regexp.Regexp;
  * requires the Jakarta Oro Package).
  *
  * <pre>
- * For jdk  &lt;= 1.3, there are two available implementations:
- *   org.apache.tools.ant.util.regexp.JakartaOroRegexp (the default)
+ * Available implementations:
+ *
+ *   org.apache.tools.ant.util.regexp.Jdk14RegexpRegexp (default)
+ *        Uses Java's built-in regular expression package
+ *
+ *   org.apache.tools.ant.util.regexp.JakartaOroRegexp
  *        Requires  the jakarta-oro package
  *
  *   org.apache.tools.ant.util.regexp.JakartaRegexpRegexp
  *        Requires the jakarta-regexp package
  *
- * For jdk &gt;= 1.4 an additional implementation is available:
- *   org.apache.tools.ant.util.regexp.Jdk14RegexpRegexp
- *        Requires the jdk 1.4 built in regular expression package.
- *
  * Usage:
  *
  *   Call Syntax:
@@ -117,12 +120,14 @@ public class ReplaceRegExp extends Task {
     private File file;
     private String flags;
     private boolean byline;
-    private Vector filesets; // Keep jdk 1.1 compliant so others can use this
+    private Union resources;
     private RegularExpression regex;
     private Substitution subs;
 
     private static final FileUtils FILE_UTILS = FileUtils.getFileUtils();
 
+    private boolean preserveLastModified = false;
+
     /**
      * Encoding to assume for the files
      */
@@ -132,7 +137,6 @@ public class ReplaceRegExp extends Task {
     public ReplaceRegExp() {
         super();
         this.file = null;
-        this.filesets = new Vector();
         this.flags = "";
         this.byline = false;
 
@@ -251,9 +255,23 @@ public class ReplaceRegExp extends Task {
      * @param set the fileset element
      */
     public void addFileset(FileSet set) {
-        filesets.addElement(set);
+        addConfigured(set);
     }
 
+    /**
+     * Support arbitrary file system based resource collections.
+     *
+     * @since Ant 1.8.0
+     */
+    public void addConfigured(ResourceCollection rc) {
+        if (!rc.isFilesystemOnly()) {
+            throw new BuildException("only filesystem resources are supported");
+        }
+        if (resources == null) {
+            resources = new Union();
+        }
+        resources.add(rc);
+    }
 
     /**
      * A regular expression.
@@ -286,6 +304,15 @@ public class ReplaceRegExp extends Task {
         return subs;
     }
 
+    /**
+     * Whether the file timestamp shall be preserved even if the file
+     * is modified.
+     *
+     * @since Ant 1.8.0
+     */
+    public void setPreserveLastModified(boolean b) {
+        preserveLastModified = b;
+    }
 
     /**
      * Invoke a regular expression (r) on a string (input) using
@@ -327,6 +354,7 @@ public class ReplaceRegExp extends Task {
 
         Reader r = null;
         Writer w = null;
+        BufferedWriter bw = null;
 
         try {
             if (encoding == null) {
@@ -339,8 +367,7 @@ public class ReplaceRegExp extends Task {
             }
 
             BufferedReader br = new BufferedReader(r);
-            BufferedWriter bw = new BufferedWriter(w);
-            PrintWriter pw = new PrintWriter(bw);
+            bw = new BufferedWriter(w);
 
             boolean changes = false;
 
@@ -370,8 +397,8 @@ public class ReplaceRegExp extends Task {
                                 changes = true;
                             }
 
-                            pw.print(res);
-                            pw.print('\r');
+                            bw.write(res);
+                            bw.write('\r');
 
                             linebuf = new StringBuffer();
                             // hasCR is still true (for the second one)
@@ -388,12 +415,12 @@ public class ReplaceRegExp extends Task {
                             changes = true;
                         }
 
-                        pw.print(res);
+                        bw.write(res);
                         if (hasCR) {
-                            pw.print('\r');
+                            bw.write('\r');
                             hasCR = false;
                         }
-                        pw.print('\n');
+                        bw.write('\n');
 
                         linebuf = new StringBuffer();
                     } else { // any other char
@@ -406,9 +433,9 @@ public class ReplaceRegExp extends Task {
                                 changes = true;
                             }
 
-                            pw.print(res);
+                            bw.write(res);
                             if (hasCR) {
-                                pw.print('\r');
+                                bw.write('\r');
                                 hasCR = false;
                             }
 
@@ -421,7 +448,6 @@ public class ReplaceRegExp extends Task {
                     }
                 } while (c >= 0);
 
-                pw.flush();
             } else {
                 String buf = FileUtils.safeReadFully(br);
 
@@ -431,10 +457,11 @@ public class ReplaceRegExp extends Task {
                     changes = true;
                 }
 
-                pw.print(res);
-                pw.flush();
+                bw.write(res);
             }
 
+            bw.flush();
+
             r.close();
             r = null;
             w.close();
@@ -443,17 +470,22 @@ public class ReplaceRegExp extends Task {
             if (changes) {
                 log("File has changed; saving the updated file", Project.MSG_VERBOSE);
                 try {
+                    long origLastModified = f.lastModified();
                     FILE_UTILS.rename(temp, f);
+                    if (preserveLastModified) {
+                        FILE_UTILS.setFileLastModified(f, origLastModified);
+                    }
                     temp = null;
                 } catch (IOException e) {
                     throw new BuildException("Couldn't rename temporary file "
-                                             + temp, getLocation());
+                                             + temp, e, getLocation());
                 }
             } else {
                 log("No change made", Project.MSG_DEBUG);
             }
         } finally {
             FileUtils.close(r);
+            FileUtils.close(bw);
             FileUtils.close(w);
             if (temp != null) {
                 temp.delete();
@@ -475,28 +507,13 @@ public class ReplaceRegExp extends Task {
             throw new BuildException("Nothing to replace expression with.");
         }
 
-        if (file != null && filesets.size() > 0) {
+        if (file != null && resources != null) {
             throw new BuildException("You cannot supply the 'file' attribute "
-                                     + "and filesets at the same time.");
+                                     + "and resource collections at the same "
+                                     + "time.");
         }
 
-        int options = 0;
-
-        if (flags.indexOf('g') != -1) {
-            options |= Regexp.REPLACE_ALL;
-        }
-
-        if (flags.indexOf('i') != -1) {
-            options |= Regexp.MATCH_CASE_INSENSITIVE;
-        }
-
-        if (flags.indexOf('m') != -1) {
-            options |= Regexp.MATCH_MULTILINE;
-        }
-
-        if (flags.indexOf('s') != -1) {
-            options |= Regexp.MATCH_SINGLELINE;
-        }
+        int options = RegexpUtil.asOptions(flags);
 
         if (file != null && file.exists()) {
             try {
@@ -511,16 +528,11 @@ public class ReplaceRegExp extends Task {
                 + file.getAbsolutePath() + "'", Project.MSG_ERR);
         }
 
-        int sz = filesets.size();
-
-        for (int i = 0; i < sz; i++) {
-            FileSet fs = (FileSet) (filesets.elementAt(i));
-            DirectoryScanner ds = fs.getDirectoryScanner(getProject());
-
-            String[] files = ds.getIncludedFiles();
-
-            for (int j = 0; j < files.length; j++) {
-                File f = new File(fs.getDir(getProject()), files[j]);
+        if (resources != null) {
+            for (Iterator i = resources.iterator(); i.hasNext(); ) {
+                FileProvider fp =
+                    (FileProvider) ((Resource) i.next()).as(FileProvider.class);
+                File f = fp.getFile();
 
                 if (f.exists()) {
                     try {
